@@ -21,7 +21,7 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  *              Dan Milstein <danmil@shore.net>                            *
  *              Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 481775 $                                          *
+ * Version:     $Revision: 527077 $                                          *
  ***************************************************************************/
 
 /*
@@ -68,6 +68,7 @@
 #define JK_ENV_CIPHER               ("SSL_CIPHER")
 #define JK_ENV_SESSION              ("SSL_SESSION_ID")
 #define JK_ENV_KEY_SIZE             ("SSL_CIPHER_USEKEYSIZE")
+#define JK_ENV_CERTCHAIN_PREFIX     ("SSL_CLIENT_CERT_CHAIN_")
 #define JK_ENV_WORKER_NAME          ("JK_WORKER_NAME")
 #define JK_NOTE_WORKER_NAME         ("JK_WORKER_NAME")
 #define JK_NOTE_WORKER_TYPE         ("JK_WORKER_TYPE")
@@ -101,6 +102,16 @@ static mode_t xfer_mode = (S_IREAD | S_IWRITE);
 static mode_t xfer_mode = (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
 #endif
 
+/*
+ * Environment variable forward object
+ */
+typedef struct
+{
+    int has_default;
+    char *name;
+    char *value;
+} envvar_item;
+
 /*
  * Configuration object for the mod_jk module.
  */
@@ -157,6 +168,7 @@ typedef struct
     char *cipher_indicator;
     char *session_indicator;
     char *key_size_indicator;
+    char *certchain_prefix;     /* Client certificate chain prefix */
 
     /*
      * Jk Options
@@ -164,11 +176,14 @@ typedef struct
     int options;
     int exclude_options;
 
+    int strip_session;
     /*
      * Environment variables support
      */
     int envvars_in_use;
     table *envvars;
+    table *envvars_def;
+    array_header *envvar_items;
 
     server_rec *s;
 } jk_server_conf_t;
@@ -448,6 +463,9 @@ static void jk_error_exit(const char *file,
     va_end(ap);
 
     ap_log_error(file, line, level, s, res);
+    if ( s ) {
+        ap_log_error(file, line, level, NULL, res);
+    }
 
     /* Exit process */
     exit(1);
@@ -632,8 +650,34 @@ static int init_ws_service(apache_private_data_t * private_data,
                 s->ssl_cert =
                     (char *)ap_table_get(r->subprocess_env,
                                          conf->certs_indicator);
+
+                if (conf->options & JK_OPT_FWDCERTCHAIN) {
+                    array_header *t = ap_table_elts(r->subprocess_env);
+                    if (t && t->nelts) {
+                        int i;
+                        table_entry *elts = (table_entry *) t->elts;
+                        array_header *certs = ap_make_array(r->pool, 1,
+                                                            sizeof(char *));
+                        *(const char **)ap_push_array(certs) = s->ssl_cert;
+                        for (i = 0; i < t->nelts; i++) {
+                            if (!elts[i].key)
+                                continue;
+                            if (!strncasecmp(elts[i].key,
+                                             conf->certchain_prefix,
+                                             strlen(conf->certchain_prefix)))
+                            *(const char **)ap_push_array(certs) = elts[i].val;
+                        }
+                        s->ssl_cert = ap_array_pstrcat(r->pool, certs, '\0');
+                     }
+                }
+
                 if (s->ssl_cert) {
                     s->ssl_cert_len = strlen(s->ssl_cert);
+                    if (JK_IS_DEBUG_LEVEL(conf->log)) {
+                        jk_log(conf->log, JK_LOG_DEBUG,
+                               "SSL client certificate (%d bytes): %s",
+                               s->ssl_cert_len, s->ssl_cert);
+                    }
                 }
                 /* Servlet 2.3 API */
                 s->ssl_cipher =
@@ -655,25 +699,33 @@ static int init_ws_service(apache_private_data_t * private_data,
         }
 
         if (conf->envvars_in_use) {
-            array_header *t = ap_table_elts(conf->envvars);
+            const array_header *t = conf->envvar_items;
             if (t && t->nelts) {
                 int i;
-                table_entry *elts = (table_entry *) t->elts;
+                int j = 0;
+                envvar_item *elts = (envvar_item *) t->elts;
                 s->attributes_names =
                     ap_palloc(r->pool, sizeof(char *) * t->nelts);
                 s->attributes_values =
                     ap_palloc(r->pool, sizeof(char *) * t->nelts);
 
                 for (i = 0; i < t->nelts; i++) {
-                    s->attributes_names[i] = elts[i].key;
-                    s->attributes_values[i] =
-                        (char *)ap_table_get(r->subprocess_env, elts[i].key);
-                    if (!s->attributes_values[i]) {
-                        s->attributes_values[i] = elts[i].val;
+                    s->attributes_names[i - j] = elts[i].name;
+                    s->attributes_values[i - j] =
+                        (char *)ap_table_get(r->subprocess_env, elts[i].name);
+                    if (!s->attributes_values[i - j]) {
+                        if (elts[i].has_default) {
+                            s->attributes_values[i - j] = elts[i].value;
+                        }
+                        else {
+                            s->attributes_values[i - j] = "";
+                            s->attributes_names[i - j] = "";
+                            j++;
+                        }
                     }
                 }
 
-                s->num_attributes = t->nelts;
+                s->num_attributes = t->nelts - j;
             }
         }
     }
@@ -702,7 +754,7 @@ static int init_ws_service(apache_private_data_t * private_data,
                 s->headers_values[i] = ap_pstrdup(r->pool, elts[i].val);
                 s->headers_names[i] = hname;
                 if (need_content_length_header &&
-                    !strcasecmp(s->headers_values[i], "content-length")) {
+                    !strcasecmp(s->headers_names[i], "content-length")) {
                     need_content_length_header = JK_FALSE;
                 }
             }
@@ -1098,6 +1150,25 @@ static const char *jk_set_auto_alias(cmd_parms * cmd,
     return NULL;
 }
 
+/*
+ * JkStripSession directive handling
+ *
+ * JkStripSession On/Off
+ */
+
+static const char *jk_set_strip_session(cmd_parms * cmd, void *dummy, int flag)
+{
+    server_rec *s = cmd->server;
+    jk_server_conf_t *conf =
+        (jk_server_conf_t *) ap_get_module_config(s->module_config,
+                                                  &jk_module);
+
+    /* Set up our value */
+    conf->strip_session = flag ? JK_TRUE : JK_FALSE;
+
+    return NULL;
+}
+
 /*****************************************************************
  *
  * Actually logging.
@@ -1542,6 +1613,25 @@ static const char *jk_set_cipher_indicator(cmd_parms * cmd,
     return NULL;
 }
 
+/*
+ * JkCERTCHAINPrefix Directive Handling
+ *
+ * JkCERTCHAINPrefix SSL_CLIENT_CERT_CHAIN_
+ */
+
+static const char *jk_set_certchain_prefix(cmd_parms * cmd,
+                                           void *dummy, const char *prefix)
+{
+    server_rec *s = cmd->server;
+    jk_server_conf_t *conf =
+        (jk_server_conf_t *) ap_get_module_config(s->module_config,
+                                                  &jk_module);
+
+    conf->certchain_prefix = ap_pstrdup(cmd->pool, prefix);
+
+    return NULL;
+}
+
 /*
  * JkSESSIONIndicator Directive Handling
  *
@@ -1588,6 +1678,8 @@ static const char *jk_set_key_size_indicator(cmd_parms * cmd,
  *  ForwardURICompatUnparsed => Forward URI as unparsed, spec compliant but broke mod_rewrite (old TC)
  *  ForwardURIEscaped        => Forward URI escaped and Tomcat (3.3 rc2) stuff will do the decoding part
  *  ForwardDirectories       => Forward all directory requests with no index files to Tomcat
+ * +ForwardSSLCertChain      => Forward SSL certificate chain
+ * -ForwardSSLCertChain      => Don't forward SSL certificate chain
  */
 
 const char *jk_set_options(cmd_parms * cmd, void *dummy, const char *line)
@@ -1646,6 +1738,9 @@ const char *jk_set_options(cmd_parms * cmd, void *dummy, const char *line)
         else if (!strcasecmp(w, "DisableReuse")) {
             opt = JK_OPT_DISABLEREUSE;
         }
+        else if (!strcasecmp(w, "ForwardSSLCertChain")) {
+            opt = JK_OPT_FWDCERTCHAIN;
+        }
         else
             return ap_pstrcat(cmd->pool, "JkOptions: Illegal option '", w,
                               "'", NULL);
@@ -1680,11 +1775,13 @@ static const char *jk_add_env_var(cmd_parms * cmd,
         (jk_server_conf_t *) ap_get_module_config(s->module_config,
                                                   &jk_module);
 
+    conf->envvars_in_use = JK_TRUE;
 
     /* env_name is mandatory, default_value is optional.
-     * No value means set the variable to an empty string.
+     * No value means send the attribute only, if the env var is set during runtime.
      */
     ap_table_setn(conf->envvars, env_name, default_value ? default_value : "");
+    ap_table_setn(conf->envvars_def, env_name, default_value ? "1" : "0");
 
     return NULL;
 }
@@ -1704,8 +1801,8 @@ static const char *jk_set_worker_property(cmd_parms * cmd,
         (jk_server_conf_t *) ap_get_module_config(s->module_config,
                                                   &jk_module);
 
-    if (jk_map_read_property(conf->worker_properties, line, conf->log) == JK_FALSE)
-        return ap_pstrcat(cmd->temp_pool, "Invalid JkWorkerProperty ", line);
+    if (jk_map_read_property(conf->worker_properties, line, 1, conf->log) == JK_FALSE)
+        return ap_pstrcat(cmd->temp_pool, "Invalid JkWorkerProperty ", line, NULL);
 
     return NULL;
 }
@@ -1769,6 +1866,13 @@ static const command_rec jk_cmds[] = {
     {"JkMountCopy", jk_set_mountcopy, NULL, RSRC_CONF, FLAG,
      "Should the base server mounts be copied to the virtual server"},
 
+    /*
+     * JkStripSession specifies if mod_jk should strip the ;jsessionid
+     * from the unmapped urls
+     */
+    {"JkStripSession", jk_set_strip_session, NULL, RSRC_CONF, FLAG,
+     "Should the server strip the jsessionid from unmapped URLs"},
+
     /*
      * JkLogFile & JkLogLevel specifies to where should the plugin log
      * its information and how much.
@@ -1823,6 +1927,8 @@ static const command_rec jk_cmds[] = {
      "Name of the Apache environment that contains SSL client certificates"},
     {"JkCIPHERIndicator", jk_set_cipher_indicator, NULL, RSRC_CONF, TAKE1,
      "Name of the Apache environment that contains SSL client cipher"},
+    {"JkCERTCHAINPrefix", jk_set_certchain_prefix, NULL, RSRC_CONF, TAKE1,
+     "Name of the Apache environment (prefix) that contains SSL client chain certificates"},
     {"JkSESSIONIndicator", jk_set_session_indicator, NULL, RSRC_CONF, TAKE1,
      "Name of the Apache environment that contains SSL session"},
     {"JkKEYSIZEIndicator", jk_set_key_size_indicator, NULL, RSRC_CONF, TAKE1,
@@ -1838,6 +1944,8 @@ static const command_rec jk_cmds[] = {
      *  ForwardURICompat         => Forward URI normally, less spec compliant but mod_rewrite compatible (old TC)
      *  ForwardURICompatUnparsed => Forward URI as unparsed, spec compliant but broke mod_rewrite (old TC)
      *  ForwardURIEscaped        => Forward URI escaped and Tomcat (3.3 rc2) stuff will do the decoding part
+     * +ForwardSSLCertChain      => Forward SSL certificate chain
+     * -ForwardSSLCertChain      => Don't forward SSL certificate chain
      */
     {"JkOptions", jk_set_options, NULL, RSRC_CONF, RAW_ARGS,
      "Set one of more options to configure the mod_jk module"},
@@ -2105,8 +2213,10 @@ static void *create_jk_config(ap_pool * p, server_rec * s)
         c->https_indicator = NULL;
         c->certs_indicator = NULL;
         c->cipher_indicator = NULL;
+        c->certchain_prefix = NULL;
         c->session_indicator = NULL;
         c->key_size_indicator = NULL;
+        c->strip_session = JK_UNSET;
     } else {
         c->mount_file_reload = JK_URIMAP_DEF_RELOAD;
         c->log_level = JK_LOG_DEF_LEVEL;
@@ -2124,8 +2234,10 @@ static void *create_jk_config(ap_pool * p, server_rec * s)
         c->https_indicator = JK_ENV_HTTPS;
         c->certs_indicator = JK_ENV_CERTS;
         c->cipher_indicator = JK_ENV_CIPHER;
+        c->certchain_prefix = JK_ENV_CERTCHAIN_PREFIX;
         c->session_indicator = JK_ENV_SESSION;
         c->key_size_indicator = JK_ENV_KEY_SIZE;
+        c->strip_session = JK_FALSE;
     }
 
     if (!jk_map_alloc(&(c->uri_to_context))) {
@@ -2139,6 +2251,8 @@ static void *create_jk_config(ap_pool * p, server_rec * s)
 
     c->envvars_in_use = JK_FALSE;
     c->envvars = ap_make_table(p, 0);
+    c->envvars_def = ap_make_table(p, 0);
+    c->envvar_items = ap_make_array(p, 0, sizeof(envvar_item));
 
     c->s = s;
     jk_map_put(c->worker_properties, "ServerRoot", ap_server_root, NULL);
@@ -2190,6 +2304,8 @@ static void *merge_jk_config(ap_pool * p, void *basev, void *overridesv)
         overrides->certs_indicator = base->certs_indicator;
     if (!overrides->cipher_indicator)
         overrides->cipher_indicator = base->cipher_indicator;
+    if (!overrides->certchain_prefix)
+        overrides->certchain_prefix = base->certchain_prefix;
     if (!overrides->session_indicator)
         overrides->session_indicator = base->session_indicator;
     if (!overrides->key_size_indicator)
@@ -2215,6 +2331,16 @@ static void *merge_jk_config(ap_pool * p, void *basev, void *overridesv)
                 }
             }
         }
+        arr = ap_table_elts(base->envvars_def);
+        if (arr) {
+            overrides->envvars_in_use = JK_TRUE;
+            elts = (const table_entry *)arr->elts;
+            for (i = 0; i < arr->nelts; ++i) {
+                if (!ap_table_get(overrides->envvars_def, elts[i].key)) {
+                    ap_table_setn(overrides->envvars_def, elts[i].key, elts[i].val);
+                }
+            }
+        }
     }
 
     if (overrides->mount_file_reload == JK_UNSET)
@@ -2228,6 +2354,8 @@ static void *merge_jk_config(ap_pool * p, void *basev, void *overridesv)
         if (!overrides->alias_dir)
             overrides->alias_dir = base->alias_dir;
     }
+    if (overrides->strip_session == JK_UNSET)
+        overrides->strip_session = base->strip_session;
 
     return overrides;
 }
@@ -2238,17 +2366,24 @@ static int JK_METHOD jk_log_to_file(jk_logger_t *l,
     if (l &&
         (l->level <= level || level == JK_LOG_REQUEST_LEVEL) &&
          l->logger_private && what) {
-        file_logger_t *flp = l->logger_private;
+        jk_file_logger_t *flp = l->logger_private;
         int log_fd = flp->log_fd;
         size_t sz = strlen(what);
         if (log_fd >= 0 && sz) {
-            if ( write(log_fd, what, sz) < 0 )
-            {
-                ap_log_error(APLOG_MARK, APLOG_ERR | APLOG_NOERRNO,
-                             NULL,
+            if (write(log_fd, what, sz) < 0 ) {
+                ap_log_error(APLOG_MARK, APLOG_ERR | APLOG_NOERRNO, NULL,
                              "mod_jk: jk_log_to_file %s failed",
                              what);
             }
+            else {
+                char c;
+#if defined(WIN32)
+                c = '\r';
+                write(log_fd, &c, 1);
+#endif
+                c = '\n';
+                write(log_fd, &c, 1);
+            }
         }
 
         return JK_TRUE;
@@ -2278,7 +2413,7 @@ static void open_jk_log(server_rec *s, pool *p)
     int jklogfd;
     piped_log *pl;
     jk_logger_t *jkl;
-    file_logger_t *flp;
+    jk_file_logger_t *flp;
     jk_server_conf_t *conf =
         (jk_server_conf_t *) ap_get_module_config(s->module_config,
                                                   &jk_module);
@@ -2286,7 +2421,7 @@ static void open_jk_log(server_rec *s, pool *p)
     if (!s->is_virtual && !conf->log_file) {
         conf->log_file = ap_server_root_relative(p, JK_LOG_DEF_FILE);
         if (conf->log_file)
-            ap_log_error(APLOG_MARK, APLOG_INFO | APLOG_NOERRNO, s,
+            ap_log_error(APLOG_MARK, APLOG_WARNING | APLOG_NOERRNO, s,
                          "No JkLogFile defined in httpd.conf. "
                          "Using default %s", conf->log_file);
     }
@@ -2342,7 +2477,7 @@ static void open_jk_log(server_rec *s, pool *p)
     }
     conf->log_fd = jklogfd;
     jkl = (jk_logger_t *)ap_palloc(p, sizeof(jk_logger_t));
-    flp = (file_logger_t *)ap_palloc(p, sizeof(file_logger_t));
+    flp = (jk_file_logger_t *)ap_palloc(p, sizeof(jk_file_logger_t));
     if (jkl && flp) {
         jkl->log = jk_log_to_file;
         jkl->level = conf->log_level;
@@ -2395,6 +2530,34 @@ static void jk_init(server_rec * s, ap_pool * p)
                                  "JkRequestLogFormat format array NULL");
             }
             sconf->options &= ~sconf->exclude_options;
+            if (sconf->envvars_in_use) {
+                int i;
+                const array_header *arr;
+                const table_entry *elts;
+                envvar_item *item;
+                const char *envvar_def;
+
+                arr = ap_table_elts(sconf->envvars);
+                if (arr) {
+                    elts = (const table_entry *)arr->elts;
+                    for (i = 0; i < arr->nelts; ++i) {
+                        item = (envvar_item *)ap_push_array(sconf->envvar_items);
+                        if (!item)
+                            jk_error_exit(APLOG_MARK, APLOG_EMERG, srv,
+                                          p, "Memory error");
+                        item->name = elts[i].key;
+                        envvar_def = ap_table_get(sconf->envvars_def, elts[i].key);
+                        if (envvar_def && !strcmp("1", envvar_def) ) {
+                            item->value = elts[i].val;
+                            item->has_default = 1;
+                        }
+                        else {
+                            item->value = "";
+                            item->has_default = 0;
+                        }
+                    }
+                }
+            }
         }
     }
 
@@ -2407,7 +2570,7 @@ static void jk_init(server_rec * s, ap_pool * p)
 #endif
 
         if (jk_shm_file)
-            ap_log_error(APLOG_MARK, APLOG_INFO | APLOG_NOERRNO, s,
+            ap_log_error(APLOG_MARK, APLOG_WARNING | APLOG_NOERRNO, s,
                          "No JkShmFile defined in httpd.conf. "
                          "Using default %s", jk_shm_file);
     }
@@ -2419,14 +2582,9 @@ static void jk_init(server_rec * s, ap_pool * p)
                    jk_shm_name(), rc);
     }
     else
-        jk_log(conf->log, JK_LOG_ERROR, "Initializing shm:%s errno=%d",
+        jk_log(conf->log, JK_LOG_ERROR,
+               "Initializing shm:%s errno=%d. Load balancing workers will not function properly.",
                jk_shm_name(), rc);
-#if !defined(WIN32) && !defined(NETWARE)
-    if (!jk_shm_file)
-        ap_log_error(APLOG_MARK, APLOG_EMERG | APLOG_NOERRNO, s,
-                     "No JkShmFile defined in httpd.conf. "
-                     "LoadBalancer will not function properly!");
-#endif
 
     /* SREVILAK -- register cleanup handler to clear resources on restart,
      * to make sure log file gets closed in the parent process  */
@@ -2445,34 +2603,39 @@ for (i = 0; i < jk_map_size(conf->automount); i++)
 }
 */
 
-    if (!jk_map_read_properties(init_map, conf->worker_file, NULL, conf->log)) {
-
-        if (jk_map_size(init_map) == 0) {
-            ap_log_error(APLOG_MARK, APLOG_EMERG, s,
-                         "No worker file and no worker options in httpd.conf "
-                         "use JkWorkerFile to set workers");
-            return;
-        }
-
+    if ((conf->worker_file != NULL) &&
+        !jk_map_read_properties(init_map, conf->worker_file, NULL, 1, conf->log)) {
+        jk_error_exit(APLOG_MARK, APLOG_EMERG | APLOG_NOERRNO, s, p,
+                      "Error in reading worker properties from '%s'",
+                      conf->worker_file);
     }
-#if MODULE_MAGIC_NUMBER >= 19980527
-        /* Tell apache we're here */
-    ap_add_version_component(JK_EXPOSED_VERSION);
-#endif
 
     if (jk_map_resolve_references(init_map, "worker.", 1, 1, conf->log) == JK_FALSE) {
-        jk_error_exit(APLOG_MARK, APLOG_EMERG, s, p, "Error in resolving configuration references");
+        jk_error_exit(APLOG_MARK, APLOG_EMERG | APLOG_NOERRNO, s, p,
+                      "Error in resolving configuration references");
     }
 
     /* we add the URI->WORKER MAP since workers using AJP14 will feed it */
     worker_env.uri_to_worker = conf->uw_map;
     worker_env.virtual = "*";       /* for now */
     worker_env.server_name = (char *)ap_get_server_version();
-    if (wc_open(init_map, &worker_env, conf->log))
-        return;
 
-    ap_log_error(APLOG_MARK, APLOG_ERR, s,
-                 "Error while opening the workers, jk will not work");
+    if (wc_open(init_map, &worker_env, conf->log)) {
+#if MODULE_MAGIC_NUMBER >= 19980527
+        /* Tell apache we're here */
+        ap_add_version_component(JK_EXPOSED_VERSION);
+#endif
+        jk_log(conf->log, JK_LOG_INFO,
+               "%s initialized",
+               JK_EXPOSED_VERSION);
+        return;
+    }
+    else {
+        ap_log_error(APLOG_MARK, APLOG_ERR, s,
+                     "Error in creating the workers."
+                     " Please consult your mod_jk log file '%s'.", conf->log_file);
+    }
+
 }
 
 /*
@@ -2612,6 +2775,24 @@ static int jk_translate(request_rec * r)
                     }
                 }
             }
+            else if (conf->strip_session == JK_TRUE) {
+                char *jsessionid;
+                if (r->uri) {
+                    jsessionid = strstr(r->uri, JK_PATH_SESSION_IDENTIFIER);
+                    if (jsessionid) {
+                        if (JK_IS_DEBUG_LEVEL(conf->log))
+                            jk_log(conf->log, JK_LOG_DEBUG,
+                                   "removing session identifier [%s] for non servlet url [%s]",
+                                   jsessionid, r->uri);
+                        *jsessionid = '\0';
+                    }
+                }
+                if (r->filename) {
+                    jsessionid = strstr(r->filename, JK_PATH_SESSION_IDENTIFIER);
+                    if (jsessionid)
+                        *jsessionid = '\0';
+                }
+            }
         }
     }
 
@@ -2678,10 +2859,33 @@ static int jk_fixups(request_rec * r)
     return DECLINED;
 }
 
-static void exit_handler(server_rec * s, ap_pool * p)
+static void child_exit_handler(server_rec * s, ap_pool * p)
 {
     /* srevilak - refactor cleanup body to jk_generic_cleanup() */
     jk_generic_cleanup(s);
+    jk_shm_close();
+}
+
+static void child_init_handler(server_rec * s, ap_pool * p)
+{
+    int rc;
+    jk_server_conf_t *conf =
+        (jk_server_conf_t *) ap_get_module_config(s->module_config,
+                                                  &jk_module);
+
+    JK_TRACE_ENTER(conf->log);
+
+    if ((rc = jk_shm_attach(jk_shm_file, jk_shm_size, conf->log)) == 0) {
+        if (JK_IS_DEBUG_LEVEL(conf->log))
+            jk_log(conf->log, JK_LOG_DEBUG, "Attached shm:%s",
+                   jk_shm_name());
+    }
+    else
+        jk_log(conf->log, JK_LOG_ERROR, "Attaching shm:%s errno=%d",
+               jk_shm_name(), rc);
+
+    JK_TRACE_EXIT(conf->log);
+
 }
 
 
@@ -2746,8 +2950,8 @@ module MODULE_VAR_EXPORT jk_module = {
     jk_fixups,                  /* [8] fixups */
     NULL,                       /* [10] logger */
     NULL,                       /* [3] header parser */
-    NULL,                       /* apache child process initializer */
-    exit_handler,               /* apache child process exit/cleanup */
+    child_init_handler,         /* apache child process initializer */
+    child_exit_handler,         /* apache child process exit/cleanup */
     NULL                        /* [1] post read_request handling */
 #ifdef EAPI
         /*
