@@ -19,7 +19,7 @@
  * Description: In process JNI worker                                      *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Based on:                                                               *
- * Version:     $Revision: 466585 $                                           *
+ * Version:     $Revision: 501764 $                                           *
  ***************************************************************************/
 
 #if !defined(WIN32) && !defined(NETWARE) && !defined(AS400)
@@ -32,6 +32,7 @@
 #include "jk_jni_worker.h"
 #include "jk_util.h"
 
+#if !defined(JNI_VERSION_1_6)
 #if defined LINUX && defined APACHE2_SIGHACK
 #include <pthread.h>
 #include <signal.h>
@@ -1236,3 +1237,12 @@ static void detach_from_jvm(jni_worker_t * p, jk_logger_t *l)
     }
     JK_TRACE_EXIT(l);
 }
+#else
+int JK_METHOD jni_worker_factory(jk_worker_t **w,
+                                 const char *name, jk_logger_t *l)
+{
+    if (w)
+        *w = NULL;
+    return 0;
+}
+#endif /* JNI_VERSION_1_6 */
