@@ -22,7 +22,7 @@
  * Author:      Mladen Turk <mturk@apache.org>                             *
  * Author:      Rainer Jung <rjung@apache.org>                             *
  * Based on:                                                               *
- * Version:     $Revision: 485092 $                                          *
+ * Version:     $Revision: 526888 $                                          *
  ***************************************************************************/
 
 #include "jk_pool.h"
@@ -40,8 +40,8 @@
  * The load balancing code in this
  */
 
-#define JK_WORKER_USABLE(w)   ((w)->state != JK_LB_STATE_ERROR && (w)->state != JK_LB_STATE_BUSY && (w)->activation != JK_LB_ACTIVATION_STOPPED && (w)->activation != JK_LB_ACTIVATION_DISABLED)
-#define JK_WORKER_USABLE_STICKY(w)   ((w)->state != JK_LB_STATE_ERROR && (w)->activation != JK_LB_ACTIVATION_STOPPED)
+#define JK_WORKER_USABLE(w)   ((w)->state != JK_LB_STATE_ERROR && (w)->state != JK_LB_STATE_PROBE && (w)->state != JK_LB_STATE_BUSY && (w)->activation != JK_LB_ACTIVATION_STOPPED && (w)->activation != JK_LB_ACTIVATION_DISABLED)
+#define JK_WORKER_USABLE_STICKY(w)   ((w)->state != JK_LB_STATE_ERROR && (w)->state != JK_LB_STATE_PROBE && (w)->activation != JK_LB_ACTIVATION_STOPPED)
 
 static const char *lb_locking_type[] = {
     JK_LB_LOCK_TEXT_OPTIMISTIC,
@@ -65,6 +65,8 @@ static const char *lb_state_type[] = {
     JK_LB_STATE_TEXT_RECOVER,
     JK_LB_STATE_TEXT_BUSY,
     JK_LB_STATE_TEXT_ERROR,
+    JK_LB_STATE_TEXT_FORCE,
+    JK_LB_STATE_TEXT_PROBE,
     "unknown",
     NULL
 };
@@ -77,6 +79,32 @@ static const char *lb_activation_type[] = {
     NULL
 };
 
+static const char *lb_first_log_names[] = {
+    JK_NOTE_LB_FIRST_NAME,
+    JK_NOTE_LB_FIRST_VALUE,
+    JK_NOTE_LB_FIRST_ACCESSED,
+    JK_NOTE_LB_FIRST_READ,
+    JK_NOTE_LB_FIRST_TRANSFERRED,
+    JK_NOTE_LB_FIRST_ERRORS,
+    JK_NOTE_LB_FIRST_BUSY,
+    JK_NOTE_LB_FIRST_ACTIVATION,
+    JK_NOTE_LB_FIRST_STATE,
+    NULL
+};
+
+static const char *lb_last_log_names[] = {
+    JK_NOTE_LB_LAST_NAME,
+    JK_NOTE_LB_LAST_VALUE,
+    JK_NOTE_LB_LAST_ACCESSED,
+    JK_NOTE_LB_LAST_READ,
+    JK_NOTE_LB_LAST_TRANSFERRED,
+    JK_NOTE_LB_LAST_ERRORS,
+    JK_NOTE_LB_LAST_BUSY,
+    JK_NOTE_LB_LAST_ACTIVATION,
+    JK_NOTE_LB_LAST_STATE,
+    NULL
+};
+
 struct lb_endpoint
 {
     lb_worker_t *worker;
@@ -172,6 +200,10 @@ int jk_lb_get_state_code(const char *v)
         return JK_LB_STATE_BUSY;
     else if  (*v == 'e' || *v == 'E' || *v == '4')
         return JK_LB_STATE_ERROR;
+    else if  (*v == 'f' || *v == 'F' || *v == '5')
+        return JK_LB_STATE_FORCE;
+    else if  (*v == 'p' || *v == 'P' || *v == '6')
+        return JK_LB_STATE_PROBE;
     else
         return JK_LB_STATE_DEF;
 }
@@ -395,6 +427,8 @@ static int recover_workers(lb_worker_t *p,
     worker_record_t *w = NULL;
     JK_TRACE_ENTER(l);
 
+    if (p->sequence != p->s->sequence)
+        jk_lb_pull(p, l);
     for (i = 0; i < p->num_of_workers; i++) {
         w = &p->lb_workers[i];
         if (w->s->state == JK_LB_STATE_ERROR) {
@@ -444,7 +478,7 @@ static int force_recovery(lb_worker_t *p,
                 jk_log(l, JK_LOG_INFO,
                        "worker %s is marked for recovery",
                        w->s->name);
-            w->s->state = JK_LB_STATE_RECOVER;
+            w->s->state = JK_LB_STATE_FORCE;
             forced++;
         }
     }
@@ -717,7 +751,7 @@ static worker_record_t *get_most_suitable_worker(lb_worker_t * p,
     }
     if (!r) {
        jk_log(l, JK_LOG_ERROR,
-              "locking failed with errno=%d",
+              "locking failed (errno=%d)",
               errno);
         JK_TRACE_EXIT(l);
         return NULL;
@@ -790,6 +824,47 @@ static worker_record_t *get_most_suitable_worker(lb_worker_t * p,
     return rc;
 }
 
+static void lb_add_log_items(jk_ws_service_t *s,
+                             const char *const *log_names,
+                             worker_record_t *w,
+                             jk_logger_t *l)
+{
+    const char **log_values = jk_pool_alloc(s->pool, sizeof(char *) * JK_LB_NOTES_COUNT);
+    char *buf = jk_pool_alloc(s->pool, sizeof(char *) * JK_LB_NOTES_COUNT * JK_LB_UINT64_STR_SZ);
+    if (log_values && buf) {
+        /* JK_NOTE_LB_FIRST/LAST_NAME */
+        log_values[0] = w->s->name;
+        snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT64_T_FMT, w->s->lb_value);
+        /* JK_NOTE_LB_FIRST/LAST_VALUE */
+        log_values[1] = buf;
+        buf += JK_LB_UINT64_STR_SZ;
+        snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT64_T_FMT, w->s->elected);
+        /* JK_NOTE_LB_FIRST/LAST_ACCESSED */
+        log_values[2] = buf;
+        buf += JK_LB_UINT64_STR_SZ;
+        snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT64_T_FMT, w->s->readed);
+        /* JK_NOTE_LB_FIRST/LAST_READ */
+        log_values[3] = buf;
+        buf += JK_LB_UINT64_STR_SZ;
+        snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT64_T_FMT, w->s->transferred);
+        /* JK_NOTE_LB_FIRST/LAST_TRANSFERRED */
+        log_values[4] = buf;
+        buf += JK_LB_UINT64_STR_SZ;
+        snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT32_T_FMT, w->s->errors);
+        /* JK_NOTE_LB_FIRST/LAST_ERRORS */
+        log_values[5] = buf;
+        buf += JK_LB_UINT64_STR_SZ;
+        snprintf(buf, JK_LB_UINT64_STR_SZ, "%d", w->s->busy);
+        /* JK_NOTE_LB_FIRST/LAST_BUSY */
+        log_values[6] = buf;
+        /* JK_NOTE_LB_FIRST/LAST_ACTIVATION */
+        log_values[7] = jk_lb_get_activation(w, l);
+        /* JK_NOTE_LB_FIRST/LAST_STATE */
+        log_values[8] = jk_lb_get_state(w, l);
+        s->add_log_items(s, log_names, log_values, JK_LB_NOTES_COUNT);
+    }
+}
+
 static int JK_METHOD service(jk_endpoint_t *e,
                              jk_ws_service_t *s,
                              jk_logger_t *l, int *is_error)
@@ -860,11 +935,25 @@ static int JK_METHOD service(jk_endpoint_t *e,
                 jk_log(l, JK_LOG_DEBUG,
                        "service worker=%s route=%s",
                        rec->s->name, s->route);
+
+            if (p->worker->lblock == JK_LB_LOCK_PESSIMISTIC)
+                jk_shm_lock();
+            if (rec->s->state == JK_LB_STATE_RECOVER)
+                rec->s->state = JK_LB_STATE_PROBE;
+            if (p->worker->lblock == JK_LB_LOCK_PESSIMISTIC)
+                jk_shm_unlock();
+                       
             while ((!(r=rec->w->get_endpoint(rec->w, &end, l)) || !end) && (retry < p->worker->s->retries)) {
                 retry++;
                 retry_wait *=2;
+
+                if (p->worker->lblock == JK_LB_LOCK_PESSIMISTIC)
+                    jk_shm_lock();
                 if (retry_wait > JK_LB_MAX_RETRY_WAIT)
                     retry_wait = JK_LB_MAX_RETRY_WAIT;
+                if (p->worker->lblock == JK_LB_LOCK_PESSIMISTIC)
+                    jk_shm_unlock();
+
                 if (JK_IS_DEBUG_LEVEL(l))
                     jk_log(l, JK_LOG_DEBUG,
                            "could not get free endpoint for worker"
@@ -878,8 +967,12 @@ static int JK_METHOD service(jk_endpoint_t *e,
                  * as in error if the retry number is
                  * greater then the number of retries.
                  */
+                if (p->worker->lblock == JK_LB_LOCK_PESSIMISTIC)
+                    jk_shm_lock();
                 if (rec->s->state != JK_LB_STATE_ERROR)
                     rec->s->state = JK_LB_STATE_BUSY;
+                if (p->worker->lblock == JK_LB_LOCK_PESSIMISTIC)
+                    jk_shm_unlock();
                 jk_log(l, JK_LOG_INFO,
                        "could not get free endpoint for worker %s (%d retries)",
                        rec->s->name, retry);
@@ -962,8 +1055,6 @@ static int JK_METHOD service(jk_endpoint_t *e,
                 if (service_stat == JK_TRUE) {
                     rec->s->state = JK_LB_STATE_OK;
                     rec->s->error_time = 0;
-                    if (p->worker->lblock == JK_LB_LOCK_PESSIMISTIC)
-                        jk_shm_unlock();
                     rc = JK_TRUE;
                 }
                 else if (service_stat == JK_CLIENT_ERROR) {
@@ -974,8 +1065,6 @@ static int JK_METHOD service(jk_endpoint_t *e,
                     rec->s->client_errors++;
                     rec->s->state = JK_LB_STATE_OK;
                     rec->s->error_time = 0;
-                    if (p->worker->lblock == JK_LB_LOCK_PESSIMISTIC)
-                        jk_shm_unlock();
                     jk_log(l, JK_LOG_INFO,
                            "unrecoverable error %d, request failed."
                            " Client failed in the middle of request,"
@@ -993,9 +1082,6 @@ static int JK_METHOD service(jk_endpoint_t *e,
                     rec->s->errors++;
                     rec->s->state = JK_LB_STATE_ERROR;
                     rec->s->error_time = time(NULL);
-                    if (p->worker->lblock == JK_LB_LOCK_PESSIMISTIC)
-                        jk_shm_unlock();
-
                     if (is_service_error != JK_HTTP_SERVER_BUSY) {
                         /*
                         * Error is not recoverable - break with an error.
@@ -1013,6 +1099,8 @@ static int JK_METHOD service(jk_endpoint_t *e,
                                "service failed, worker %s is in error state",
                                rec->s->name);
                 }
+                if (p->worker->lblock == JK_LB_LOCK_PESSIMISTIC)
+                    jk_shm_unlock();
             }
             if ( rc == -1 ) {
                 /*
@@ -1024,42 +1112,8 @@ static int JK_METHOD service(jk_endpoint_t *e,
                            "recoverable error... will try to recover on other worker");
             }
             if (first == 1 && s->add_log_items) {
-                const char **log_names = jk_pool_alloc(s->pool, sizeof(char *) * JK_LB_NOTES_COUNT);
-                const char **log_values = jk_pool_alloc(s->pool, sizeof(char *) * JK_LB_NOTES_COUNT);
-                char *buf = jk_pool_alloc(s->pool, sizeof(char *) * JK_LB_NOTES_COUNT * JK_LB_UINT64_STR_SZ);;
                 first = 0;
-                if (log_names && log_values && buf) {
-                    log_names[0] = JK_NOTE_LB_FIRST_NAME;
-                    log_values[0] = prec->s->name;
-                    snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT64_T_FMT, prec->s->lb_value);
-                    log_names[1] = JK_NOTE_LB_FIRST_VALUE;
-                    log_values[1] = buf;
-                    buf += JK_LB_UINT64_STR_SZ;
-                    snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT64_T_FMT, prec->s->elected);
-                    log_names[2] = JK_NOTE_LB_FIRST_ACCESSED;
-                    log_values[2] = buf;
-                    buf += JK_LB_UINT64_STR_SZ;
-                    snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT64_T_FMT, prec->s->readed);
-                    log_names[3] = JK_NOTE_LB_FIRST_READ;
-                    log_values[3] = buf;
-                    buf += JK_LB_UINT64_STR_SZ;
-                    snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT64_T_FMT, prec->s->transferred);
-                    log_names[4] = JK_NOTE_LB_FIRST_TRANSFERRED;
-                    log_values[4] = buf;
-                    buf += JK_LB_UINT64_STR_SZ;
-                    snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT32_T_FMT, prec->s->errors);
-                    log_names[5] = JK_NOTE_LB_FIRST_ERRORS;
-                    log_values[5] = buf;
-                    buf += JK_LB_UINT64_STR_SZ;
-                    snprintf(buf, JK_LB_UINT64_STR_SZ, "%d", prec->s->busy);
-                    log_names[6] = JK_NOTE_LB_FIRST_BUSY;
-                    log_values[6] = buf;
-                    log_names[7] = JK_NOTE_LB_FIRST_ACTIVATION;
-                    log_values[7] = jk_lb_get_activation(rec, l);
-                    log_names[8] = JK_NOTE_LB_FIRST_STATE;
-                    log_values[8] = jk_lb_get_state(rec, l);
-                    s->add_log_items(s, log_names, log_values, JK_LB_NOTES_COUNT);
-                }
+                lb_add_log_items(s, lb_first_log_names, prec, l);
             }
         }
         else {
@@ -1110,41 +1164,7 @@ static int JK_METHOD service(jk_endpoint_t *e,
         rc = JK_FALSE;
     }
     if (prec && s->add_log_items) {
-        const char **log_names = jk_pool_alloc(s->pool, sizeof(char *) * JK_LB_NOTES_COUNT);
-        const char **log_values = jk_pool_alloc(s->pool, sizeof(char *) * JK_LB_NOTES_COUNT);
-        char *buf = jk_pool_alloc(s->pool, sizeof(char *) * JK_LB_NOTES_COUNT * JK_LB_UINT64_STR_SZ);;
-        if (log_names && log_values && buf) {
-            log_names[0] = JK_NOTE_LB_LAST_NAME;
-            log_values[0] = prec->s->name;
-            snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT64_T_FMT, prec->s->lb_value);
-            log_names[1] = JK_NOTE_LB_LAST_VALUE;
-            log_values[1] = buf;
-            buf += JK_LB_UINT64_STR_SZ;
-            snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT64_T_FMT, prec->s->elected);
-            log_names[2] = JK_NOTE_LB_LAST_ACCESSED;
-            log_values[2] = buf;
-            buf += JK_LB_UINT64_STR_SZ;
-            snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT64_T_FMT, prec->s->readed);
-            log_names[3] = JK_NOTE_LB_LAST_READ;
-            log_values[3] = buf;
-            buf += JK_LB_UINT64_STR_SZ;
-            snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT64_T_FMT, prec->s->transferred);
-            log_names[4] = JK_NOTE_LB_LAST_TRANSFERRED;
-            log_values[4] = buf;
-            buf += JK_LB_UINT64_STR_SZ;
-            snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT32_T_FMT, prec->s->errors);
-            log_names[5] = JK_NOTE_LB_LAST_ERRORS;
-            log_values[5] = buf;
-            buf += JK_LB_UINT64_STR_SZ;
-            snprintf(buf, JK_LB_UINT64_STR_SZ, "%d", prec->s->busy);
-            log_names[6] = JK_NOTE_LB_LAST_BUSY;
-            log_values[6] = buf;
-            log_names[7] = JK_NOTE_LB_LAST_ACTIVATION;
-            log_values[7] = jk_lb_get_activation(prec, l);
-            log_names[8] = JK_NOTE_LB_LAST_STATE;
-            log_values[8] = jk_lb_get_state(prec, l);
-            s->add_log_items(s, log_names, log_values, JK_LB_NOTES_COUNT);
-        }
+        lb_add_log_items(s, lb_last_log_names, prec, l);
     }
 
     JK_TRACE_EXIT(l);
@@ -1337,7 +1357,7 @@ static int JK_METHOD init(jk_worker_t *pThis,
     JK_INIT_CS(&(p->cs), i);
     if (i == JK_FALSE) {
         jk_log(log, JK_LOG_ERROR,
-               "creating thread lock errno=%d",
+               "creating thread lock (errno=%d)",
                errno);
         JK_TRACE_EXIT(log);
         return JK_FALSE;
